#!/usr/bin/perl
#=======================
# (c)2004 Nick Temple <nickt@nicktemple.com>
# Released under the GPL: http://www.gnu.org/copyleft/gpl.html
#
# Usage:
#   newtheme.pl newtheme-name <base-theme>
#   the newtheme-name is the name of your new theme
#   base theme is the base-theme to copy, by default, themestart
#
# NOTES:
# * This was created on Win2k with ActiveState Perl 807, (perl, v5.8.1 built for MSWin32-x86-multi-thread)
#   cygwin installed.  This can lead to some "weird" convolutions of Windows/Unix commands.  It should
#   work on most unixes, however, and NT/Windows with the cygwin tools.
# * Very little error checking is done.
# * If you you use a name for your theme that appears in a file, it will be replaced. This can be bad.
# * The assumption is made that you will be using the ThemeTemplate class.  To change this,
#   you must modify ./src/resources/nukes-YOURTHEME-sar/META-INF/jboss-service.xml
#   (look for code=). You may want to modify ./src/main/org/jboss/nukes/theme/ThemeThemeStart.java,
#   Which is not used in the current implementation.

$NEW  = $ARGV[0];
$BASE = $ARGV[1] || "themestart";
$LOG = "1";

$usage = "newtheme.pl newtheme-name <base-theme>\n";
print "createTheme: $NEW\n";

# Sanity check
die ($usage) unless $NEW;
die ("Fatal: Missing base theme $BASE.") unless -e $BASE;
die ("Fatal: Theme $NEW already exists.") if -e $NEW;

open (LOG, ">$NEW.log") if $LOG;
# Copy files
# TODO: make platform independent
run("cp -r $BASE $NEW");
$files = run("find $NEW");
@files = split /\n/, $files;
print "===============\n";

# Here we do a search and replace on every file
foreach (@files) {
  chomp;
  next if /^$/;
  next unless -f $_;
  next if /images/;
  next if /\,gif$/;
  next if /\.jpg$/;
  next if /\.class$/;

  my $path = $_;
  print "s";
  srp($path, $BASE, $NEW);


  # Change the filename, if needed  
  my $newname = newname($path, $BASE, $NEW);
  unless ($path eq $newname) {
     print "f";
     mv($path, $newname);
  }
}

# Rename directories, as needed
foreach (reverse(@files)) {
  chomp;
  next if /^$/;
  next unless -d $_;

  my $path = $_;

  my $newname = newname($path, $BASE, $NEW);
  unless ($path eq $newname) {
     print "d";
     mv($path, $newname);
  }
}
close(LOG) if $LOG;
print "\nDone. Please check output log $NEW.log for errors and the\n";
print "output directory for sanity, then run build.\n";
print "You can then deploy by dropping the .ear file into your Nukes/JBOSS server folder.\n\n";

#=========

sub newname {
  my ($path, $BASE, $NEW) = @_;

  my @path = split /\//, $path;
  my $fname = pop(@path);
  $fname =~ s/$BASE/$NEW/;
  push (@path, $fname);

  return join("/", @path);
  
}
   
sub srp {
  my ($file, $old, $new) = @_;
  my $out = "";

  open (IN, $file);
  while (<IN>) {
     s/$old/$new/g;
     $out .= $_;
  }
  close (IN);
  open  (OUT, ">$file");
  print OUT $out;
  close (OUT);
  
  return $out;  
}


sub mv {
  my ($src, $dest) = @_;
  run("mv $src $dest");
}

sub run {
  my $cmd = shift;
  print LOG "$cmd\n" if $LOG;
  my $out = `$cmd`;
  print LOG "$out\n" if $LOG;
  return $out;
}

